@@ -23,37 +23,37 @@ logger = logging.getLogger(__name__)
 class Underwriter(object):
     """
     The ``Underwriter`` class manages the creation of Aggregate and Portfolio objects, and
-    maintains a database of standard Severity (curves) and Aggregate (unit or line level) objects.
-    The ``Underwriter`` knows about all the business that is written!
+    maintains a database of standard Severity (curves) and Aggregate (unit or line level) objects
+    called the knowledge base.
 
-    * Handles persistence to and from agg files
-    * Is interface into program parser
-    * Handles safe lookup from database for parser
+    - Handles persistence to and from agg files
+    - Is interface into program parser
+    - Handles safe lookup from the knowledge for parser
 
-    Objects have a kind (agg, port, or sev) and a name. E.g. agg MyAgg ... has kind agg and name MyAgg.
-    They have a representation as a program. When the program is interpreted it produces a string spec
+    Objects have a kind and a name. The kind is one of 'sev', 'agg' or 'port'. The name is a string.
+    They have a representation as a program. When the program is interpreted it produces a dictionary spec
     that can be used to create the object. The static method factory can create any object from the
     (kind, name, spec, program) quartet, though, strictly, program is not needed.
 
     The underwriter knowledge is stored in a dataframe indexed by kind and name with columns
     spec and program.
-
     """
 
     def __init__(self, name='Rory', databases=None, update=False, log2=10, debug=False):
         """
-        Create underwriter object.
+        Create an underwriter object. The underwriter is the interface to the knowledge base
+        of the aggregate system. It is the interface to the parser and the interpreter, and
+        to the database of curves, portfolios and aggregates.
 
-        :param name: Name of the underwriter object. Default is named after Rory Cline, the best
-        underwriter I know.
-        :param databases: if None: nothing loaded; if 'default' (installed) or 'site' (user,
-        in ~/aggregate/databases) database *.agg files in default or site directory are loaded.
-        If 'all' both default and site databases loaded. A string refers to a single database;
-        an interable of strings is also valid. See `read_database` for search path.
-        :param update: If True objects are updated after being created.
-        :param log2: Default log2 value.
-        :param debug: run parser in debug mode
-        that are within Portfolios.
+        :param name: name of underwriter. Defaults to Rory, after Rory Cline, the best underwriter
+            I know and a supporter of an analytic approach to underwriting.
+        :param databases: name or list of database files to read in on creation. if None: nothing loaded; if
+            'default' (installed) or 'site' (user, in ~/aggregate/databases) database \\*.agg files in default or site
+            directory are loaded. If 'all' both default and site databases loaded. A string refers to a single database;
+            an interable of strings is also valid. See `read_database` for search path.
+        :param update: if True, update database files with new objects.
+        :param log2: log2 of number of buckets in discrete representation.  10 is 1024 buckets.
+        :param debug: if True, print debug messages.
         """
 
         self.name = name
@@ -68,20 +68,22 @@ class Underwriter(object):
         # knowledge - accounts and line known to the underwriter
         self._knowledge = pd.DataFrame(columns=['kind', 'name', 'spec', 'program'], dtype=object).set_index(
             ['kind', 'name'])
+
         if databases == 'all':
             databases = ['default', 'site']
         elif type(databases) == str:
             databases = [databases]
-        # default_dir is installed by pip and contain installation files
-        default_dir = Path(__file__).parent / 'agg'
-        # site dir is in Users's home directory and stores their files
-        site_dir = Path.home() / 'aggregate/databases'
 
-        self.site_dir = site_dir
+        # default_dir is installed by pip and contain installation files
+        self.default_dir = Path(__file__).parent / 'agg'
+
+        # site dir is in Users's home directory and stores their files
+        self.site_dir = Path.home() / 'aggregate/databases'
         # check site dir exists
         self.site_dir.mkdir(parents=True, exist_ok=True)
 
-        self.default_dir = default_dir
+        self.template_dir = self.default_dir.parent / 'templates'
+        self.template_dir.mkdir(parents=True, exist_ok=True)
 
         # make sure all database entries are stored:
         if databases is None:
@@ -91,17 +93,20 @@ class Underwriter(object):
         if 'default' in databases:
             # add all databases in default_dir
             databases.remove('default')
-            for fn in default_dir.glob('*.agg'):
+            for fn in self.default_dir.glob('*.agg'):
                 self.read_database(fn)
 
         if 'site' in databases:
             # add all user databases
             databases.remove('site')
-            databases += list(site_dir.glob('*.agg'))
+            databases += list(self.site_dir.glob('*.agg'))
 
         for fn in databases:
             self.read_database(fn)
 
+        # ?! ensure description prints correctly. A bit cheaky.
+        pd.set_option('display.max_colwidth', 100)
+
     def read_database(self, fn):
         """
         read database of curves, aggs and portfolios. These can live in the default directory
@@ -133,11 +138,14 @@ class Underwriter(object):
 
         try:
             program = db_path.read_text(encoding='utf-8')
-        except:
+        except Exception as e:
             logger.error(f'Error reading requested database {db_path.name}. Ignoring.')
         else:
             # read in, parse, save to sev/agg/port dictionaries
             # throw away answer...not creating anything
+            # get rid of cosmetic spaces, but keep newline tabs (2 or more spaces)
+            program = re.sub('^  +', '\t', program, flags=re.MULTILINE)
+            program = re.sub(' +', ' ', program)
             logger.info(f'Reading database {fn}...')
             n = len(self._knowledge)
             self.interpret_program(program)
@@ -180,6 +188,15 @@ class Underwriter(object):
             else:
                 raise KeyError(f'Error: no unique object found matching {item}. Found {len(rows)} objects.')
 
+    def __repr__(self):
+        s = []
+        s.append(f'Underwriter   {self.name}')
+        s.append(f'Knowledge     {len(self._knowledge)} programs')
+        for k in ['log2', 'update', 'debug', 'site_dir', 'default_dir']:
+            s.append(f'{k:<14s}{getattr(self, k)}')
+        # s.append(super().__repr__())
+        return '\n'.join(s)
+
     def _repr_html_(self):
         s = [f'<p><h3>Underwriter {self.name}</h3>',
              f'Knowledge contains {len(self._knowledge)} programs. '
@@ -254,9 +271,7 @@ class Underwriter(object):
         3. Replace sev.name, agg.name and port.name references with their objects.
         5. If update set, update all created objects.
 
-        Sample input
-
-        ::
+        Sample input::
 
             port MY_PORTFOLIO
                 agg Line1 20  loss 3 x 2 sev gamma 5 cv 0.30 mixed gamma 0.4
@@ -269,11 +284,10 @@ class Underwriter(object):
 
         Reasonable kwargs:
 
-        * **bs**
-        * **log2**
-        * **update** overrides class default
         * **add_exa** should port.add_exa add the exa related columns to the output?
 
+        :param log2:
+        :param bs:
         :param portfolio_program:
         :param update: override class default
         :param kwargs: passed to object's update method if update==True
@@ -436,7 +450,9 @@ class Underwriter(object):
         :return:
         """
 
-        kind, name = buildinid.split('.')
+        # allow for sev.WC.1 name
+        kind, *name = buildinid.split('.')
+        name = '.'.join(name)
         try:
             # lookup in Underwriter
             answer = self[(kind, name)]
@@ -462,7 +478,7 @@ class Underwriter(object):
         # set logger_level for all aggregate loggers
         logger_level(level)
 
-    def build(self, program, update=True, log2=0, bs=0, log_level=None, **kwargs):
+    def build(self, program, update=True, log2=0, bs=0, recommend_p=0.999, log_level=None, **kwargs):
         """
         Convenience function to make work easy for the user. Intelligent auto updating.
         Detects discrete distributions and sets ``bs = 1``.
@@ -474,9 +490,10 @@ class Underwriter(object):
         :param program:
         :param update: build's update
         :param log2: 0 is default: Estimate log2 for discrete and self.log2 for all others. Inupt value over-rides
-        and cancels discrete computation (good for large discrete outcomes where bucket happens to be 1.)
+            and cancels discrete computation (good for large discrete outcomes where bucket happens to be 1.)
         :param bs:
-        :param log_level:
+        :param log_level: temporary log(ger) level for this build
+        :param recommend_p: passed to recommend bucket functions. Increase (closer to 1) for thick tailed distributions.
         :param kwargs: passed to update, e.g., padding. Note force_severity=True is applied automatically
         :return: created object(s)
         """
@@ -523,7 +540,7 @@ class Underwriter(object):
                     else:
                         log2_ = log2
                     if bs == 0:
-                        bs_ = round_bucket(answer.object.recommend_bucket(log2_))
+                        bs_ = round_bucket(answer.object.recommend_bucket(log2_, p=recommend_p))
                     else:
                         bs_ = bs
                     logger.info(f'({answer.kind}, {answer.name}): Normal mode, using bs={bs_} (1/{1/bs_}) and log2={log2_}')
@@ -540,6 +557,8 @@ class Underwriter(object):
                 # figure stuff
                 if log2 == -1:
                     log2_ = 13
+                elif log2 == 0:
+                    log2_ = self.log2
                 else:
                     log2_ = log2
                 if bs == 0:
@@ -589,12 +608,12 @@ class Underwriter(object):
         filename is a string or Path. If a csv it is read into
         a dataframe, with the first column used as index. If it
         is an agg file (e.g. an agg database), it is preprocessed
-        to remove comments and replace \n\t agg with a space, then
+        to remove comments and replace \\n\\t agg with a space, then
         split on new lines and converted to a dataframe.
         Other file formats are rejected.
 
-        These methods are called interpreter_... rather than
-        interpret_... because they are for testing and debugging
+        These methods are called interpreter\_... rather than
+        interpret\_... because they are for testing and debugging
         the interpreter, not for actually interpreting anything!
 
         """
@@ -700,7 +719,15 @@ class Underwriter(object):
         df_out.index.name = 'index'
         return df_out
 
-    def show(self, regex, kind='agg', plot=True, show=True, logger_level=30):
+    def qshow(self, regex):
+        """
+        Wrapper for show to just list elements in knowledge that match ``regex``.
+
+        """
+
+        return self.show(regex, kind='', plot=False, describe=False)
+
+    def show(self, regex, kind='agg', plot=True, describe=True, logger_level=30):
         """
         Create from knowledge by name or match to name.
         Optionally plot. Returns the created object plus dataframe with more detailed information.
@@ -709,31 +736,38 @@ class Underwriter(object):
 
         Eg regex = "A.*[234] for A...2, 3 and 4.
 
+        See ``qshow`` for a wrapper that just returns the matches, with no object
+        creation or plotting.
+
         Examples.
         ::
+
             from aggregate.utilities import pprint
             # pretty print all prgrams starting A; no object creation
-            build.show('^A.*', 'agg', False, False).program.apply(lambda x: pprint(x, html=True));
+            build.show('^A.*', 'agg', False, False).program.apply(pprint);
 
             # build and plot A..234
             ans, df = build.show('^A.*')
 
         :param regex: for filtering name
         :param kind: the kind of object, port, agg, etc.
-        :param plot:
+        :param plot:    if True, plot   (default True)
+        :param describe: if True, print the describe dataframe
         :param logger_level: work silently!
         :return: dictionary of created objects and DataFrame with info about each.
         """
         # too painful getting the one thing out!
         ans = []
+
         # temp logger level
         lm = LoggerManager(logger_level)
+
         if kind is None or kind == '':
             df = self.knowledge.droplevel('kind').filter(regex=regex, axis=0).copy()
         else:
             df = self.knowledge.loc[kind].filter(regex=regex, axis=0).copy()
 
-        if plot is False and show is False:
+        if plot is False and describe is False:
             # just act like a filtered listing on knowledge
             return df.sort_values('name')
 
@@ -755,21 +789,26 @@ class Underwriter(object):
             except NotImplementedError:
                 logger.error(f'skipping {n}...element not implemented')
             else:
-                if show:
-                    display(a)
+                if describe:
+                    display(a.describe)
                     display(HTML('<h4>Program</h4>'))
-                    pprint(p, split=60, html=True)
+                    pprint(p)
                 if plot is True:
-                    a.plot(figsize=(10, 3))
+                    a.plot(figsize=(8, 2.4))
                     display(HTML('<h4>Density and Quantiles</h4>'))
                     show_fig(a.figure, format='svg')
-                if show:
+                if describe:
                     display(HTML('<br>'))
                 # info
-                m, cv = a.describe.loc['Agg', ['Est E(X)', 'Est CV(X)']]
+                if isinstance(a, Portfolio):
+                    m, cv = a.describe.loc[('total', 'Agg'), ['Est E[X]', 'Est CV(X)']]
+                elif isinstance(a, Aggregate):
+                    m, cv = a.describe.loc['Agg', ['Est E[X]', 'Est CV(X)']]
+                else:
+                    m = cv = np.nan
                 df.loc[n, ['log2', 'bs', 'agg_m', 'agg_cv', 'agg_sd',
-                            'emp_m', 'emp_cv', 'emp_sd']] =\
-                        (a.log2, a.bs, a.agg_m, a.agg_cv, a.agg_sd, m, cv, '')
+                            'emp_m', 'emp_cv', 'emp_sd']] = (a.log2, a.bs, a.agg_m, a.agg_cv,
+                                                             a.agg_sd, m, cv, '')
         # if only one item, return it...much easier to use
         if len(ans) == 1: ans = a
         return ans, df
@@ -908,6 +947,4 @@ class Underwriter(object):
 # self = dbuild = None
 logger_level(30)
 build = Underwriter(databases='test_suite', update=True, debug=False, log2=16)
-# differentiate later!
-actuary_build = student_build = capital_build = dev_build = build
 debug_build = Underwriter(name='Debug', update=True, debug=True, log2=13)
